/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

package ShoeSearch;

import SHOE.*;
import ShoeKb.*;
import UserInt.ApplType;
import UserInt.BorderedPanel;
import UserInt.GridBagCreator;
import UserInt.LightComponentScroller;
import UserInt.Notification;
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

/** This is a variation of the ShoeSearch application that uses SHOE
  ontologies but no other SHOE information. Essentially, the ontology
  provides a form in which users can input their queries. The major 
  differences is that the "Query" button automatically issues a query
  to a search engine, and the only button along the bottom is 
  "Advanced Query" which performs the same action as "Web Search" does
  in the original application.
*/
/* NOTE: The only reason this needs to connect to a KB is to get
         a catalog of available ontologies. */
public class OntOnly extends SS_Frame implements WebSearchContainer
{

  public static void main (String[] args)
  {
      ApplType.setApplication();

      OntOnly app = new OntOnly(DEFAULT_HOST, DEFAULT_PORT, 
				DEFAULT_KB_NAME, null);
      app.show();
  }

  public OntOnly (String kbHost, int kbPort, String kbName,
		  Applet applet)
  {
      super(kbHost, kbPort, kbName, applet);
  }

  /* Create panels and widgets. We override the parent method so that
     the "Web Search" button becomes an "Advanced Query" button, and
     the "Get Page" and "Add To Query" buttons are removed.
  */
  void setupComponents (int cid)
  {
    Label l = new Label ("Ontology:");
    ontChoice = new Choice();
    BorderedPanel catPanel = new BorderedPanel(3,4,0,4);
    setupCatPanel (catPanel);

    qwin = new QueryPanel (ontman);
    sqwin = new LightComponentScroller(qwin);   //"scrolling qwin"
    sqwinPanel = new BorderedPanel(4,4,4,4);
    sqwinPanel.setLayout (new BorderLayout());
    sqwinPanel.add ("Center", sqwin);
    sqwinPanel.setHasBorder();

    b_Query = new Button("Query");
    b_WebSearch = new Button("Advanced Query");
    statusBar = new Label("                                                  ");

    GridBagCreator   gbcreat = new GridBagCreator (this);

    gbcreat.addComponent (l,           0,  0,  1,  1,    0,   0,
                       GridBagConstraints.BOTH, 0, 0);
    gbcreat.addComponent (ontChoice,   1,  0,  1,  1,    0,   0,
                       GridBagConstraints.BOTH, 0, 0);
    gbcreat.addComponent (catPanel,    0,  1,  2,  1,   10,  90,
                       GridBagConstraints.BOTH, 0, 0);
    gbcreat.addComponent (sqwinPanel,  2,  1,  4,  1,   90,   0,
                       GridBagConstraints.BOTH, 0, 0);
    gbcreat.addComponent (b_Query,     2,  3,  1,  1,    0,   0,
                       GridBagConstraints.VERTICAL, 0, 0);
    gbcreat.addComponent (b_WebSearch, 3,  3,  1,  1,    0,   0,
                       GridBagConstraints.BOTH, 0, 0);
    gbcreat.addComponent (statusBar,   0,  4,  GridBagConstraints.REMAINDER, 1,
                       0, 0,
                       GridBagConstraints.BOTH, 0, 0);

    /* We create the following but don't add it to this component. This
       way, calls to reset it from SS_Frame won't result in errors. */
    resWin = new ResultsWin (this, context);

    setBackground(Color.lightGray);
  }


  /** Respond to menu events
  */
  public boolean handleEvent (Event event)
  {
    if (event.id == Event.WINDOW_DESTROY) {
      exit();
      return true;
    }
    else if (event.target instanceof MenuItem) {
      MenuItem menu = (MenuItem)event.target; 
      MenuItem pmenu = (MenuItem)menu.getParent();
      if (event.id == Event.ACTION_EVENT) {
         if (pmenu.getLabel().equals(MI_FILE)) {
            if (MI_EXIT.equals(event.arg))
               exit();
            else  if (MI_FWD_WS.equals(event.arg))
            {
               if (doWebSearch != null)
	               doWebSearch.toFront();
            }
            else  if (MI_FWD_WSDEP.equals(event.arg))
            {
               if (doWebSearch != null)
                  doWebSearch.assocWinToFront();
            }
         }
      }
    }
    else if (event.id == Event.LIST_SELECT) {
      if (event.target == catList) {
         // System.out.println("Selected catList");
      }
    }
    return super.handleEvent(event);
  }

  /** Respond to button presses and choice selections
  */
  public boolean action (Event evt, Object arg)
  {
    if (evt.target instanceof Button)
    {
        if (evt.target == b_Select)
        {
	  setCursor(Frame.WAIT_CURSOR);
	  resWin.newEmptyMultiColumn();           // clear results panel

	  SHOE_Obj obj = (SHOE_Obj)catList.getSelectedObject();
	  qwin.hide();
          qwin.changeSubject (contextOnt, obj.getKey());

	  /* We want to make sure the scroll pane will be re-laid out. A
	     call to validate() will not resize/reposition an object
	     and its components unless it has been marked invalid. Note
	     that Java 1.1 Container's invalidate() method overrides
	     that of Component. */
	  if (ApplType.getIsVersion1_0()) {
	    ((Component)sqwin).invalidate();     
	    ((Component)sqwinPanel).invalidate(); 
	  }
	  else {
	    sqwin.invalidate();     
	    sqwinPanel.invalidate(); 
	  }

          int  qwinSize = qwin.getRowSize();
          if (qwinSize > 0)
             sqwinPanel.setVisibleComponents();
                //this is how sqwinPanel knows when to draw its border

	  if (disableEmptyProps == true) {
	    for (int i = 0; i < qwinSize; i++)
	      {
		int numResults = doFindQueryForNumAnswers(i);
		if (numResults == 0)
		  qwin.disableRow(i);
		//for any fields with no data anyway, their rows are disabled.
	      }
	  }

          sqwin.scrollTo(0,0);
          // sqwin.resizeScrollbars();
          // sqwin.validate();

	  qwin.show();
          validate();
          repaint();
	  setCursor(Frame.DEFAULT_CURSOR);
          return true;
        }
        else if (evt.target == b_Query)
        {             // issue a query command
          doWebQuery();
          return true;
        }
        else if (evt.target == b_WebSearch)
        {
            //only create a new win if one of this type does not currently exist
            if (doWebSearch == null)
            {
               doWebSearch = new WebSearch (this,
                           TITLE + " -- Web Search", context, qwin);
                   // doWebSearch.resize (put width here, put height here);
                   //is 'show()' in Window class or Dialog class?
               if (ApplType.getIsVersion1_0())
               {
                   // System.out.println ("Version 1.0");
                   ((Window)doWebSearch).show();
               }
	            else
	            {
                   // System.out.println ("Version 1.1 or higher");
                   doWebSearch.show();
               }
            }
	         else    //put the existing window on top of stack
               doWebSearch.toFront();

            return true;
        }
    }
    else if (evt.target == ontChoice)
    {
        // System.out.println("Selected ontChoice");
        setCursor(Frame.WAIT_CURSOR);
        changeContext(ontChoice.getSelectedIndex());
        setCursor(Frame.DEFAULT_CURSOR);

        return true;
    }

    return false;
  }


  /** Issues the query to last engine selected in the Web Search window
    (defaults to Alta Vista). To do this, we create a new WebSearch
    object if necessary.
    */
  public void doWebQuery()
  {
    String webQStr;
    String urlAndQStr;
    URL    webPg;

    if (context == null) {
      Notification note;
      String msg = "A web query can only be issued when run as an applet " +
	" in a browser.";
      note = new Notification(this, "Error", msg);
      note.showWorkaround();
      return;
    }

    if (doWebSearch == null) {
      doWebSearch = new WebSearch (this, TITLE + " -- Web Search",
				   context, qwin);
      doWebSearch.hide();
    }
    
    webQStr = qwin.makeWebQuery();
    doWebSearch.setQueryString(webQStr);

    urlAndQStr = doWebSearch.getWebQuery();
    try    {
      webPg = new URL (urlAndQStr);
      //System.out.println ("Web Query URL:  "
      //      + webPg.toString());
      
      context.showDocument (webPg, WebSearch.WEB_TARGET);
    } catch  (MalformedURLException  mue)  {
      System.out.println ("Problem with query: Cannot retreive");
    }
  }

}
