/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

/*
  Change History:
    10/26/00, JDH: Used accessor methods for fields in Ontology and DefRelation
*/

package ShoeSearch;

import SHOE.*;
import ShoeKb.*;
import UserInt.LightComponentScroller;
import UserInt.Utilities;
import java.awt.*;
import java.util.*;

/** Provides a form fill-in interface for specifying a query.
*/
public class QueryPanel extends Panel
                        implements WebQueryString
{
  OntManager ontman;            // ontology cache
             /* do we really need an OntManager? */
  Ontology   ontol;             // ontology object

  String    subjectCat;         // name of category

  int       invRelsIdx;         // index of the first inverse relation
  String    relname[];          // names of relations in form
  Vector    relList;            // the current relation objects for this
                                // 'subjectCat'
  Label     l_fld[];            // labels of form fields
  TextField fldText[];          // entry fields for form
  Button    bFind[];            // buttons to find values for fields;
                                //  'Find' actions handled in window containing
                                //  this object.
  Checkbox  cbShow[];           // checkboxes to indicate showing the field

  int       relCount;           // number of items in the arrays

  static final String  URL_PREFIX   = "http:";
  static final int     TEXTF_SIZE = 30;
  static final String  FIND_BUTTON_LABEL = "Find";
  static final String  CHKBOX_BUTTON_LABEL = "Show";

  static final int     PREFERRED_HEIGHT = 200;
  static final int     PREFERRED_WIDTH  = 400;

  /*  not currently used...
  static final String  MOVETEXT_BUTTON_LABEL = "Copy";
  static final int     VAR_TRUNC_SIZE = 4;
  */
  static final String  STD_PARKA_KEY_VAR = "?K";
  static final String  STD_PARKA_KEYNAME_VAR = "?KN";
  static final String  STD_PARKA_VAR_PREFIX = "?X";
  static final String  STD_PARKA_NAMEVAR_PREFIX = "?N";

  /* Methods */


  public QueryPanel (OntManager ontman)
  {
    this.ontman = ontman;
    setLayout(new GridBagLayout());
    ontol = null;
  }

  /** The component prefers to be its actual size or to reserve enough
    screen real estate to eventually hold a list of query fields. 
    We call preferredLayoutSize() of the layout manager and
    then determine if it needs to be bigger. */
  /* NOTE: to use this function, we must fix LightComponentScroller to
     return the actual size of the scroll area instead of the preferred
     size in getScrollAreaSize().
  /*
  public Dimension preferredSize() {

    Dimension prefSize;
    
    prefSize = getLayout().preferredLayoutSize(this);
    if (PREFERRED_HEIGHT > prefSize.height);
      prefSize.height = PREFERRED_HEIGHT;
    if (PREFERRED_WIDTH > prefSize.width)
      prefSize.width = PREFERRED_WIDTH;

    // System.out.println("Preferred Size = " + prefSize);

    return prefSize;
  }
  */

  /** Update the panel to show fields that are applicable to a new
    subject category. These fields are the relations that
    the given category can fill the first argument position of.
  */
  public void changeSubject (Ontology ont, String catname)
  {
    Label        heading;
    Def_Category dcat;
    Unique_Elt   elmt_id;
    Def_Relation drel;
    int   i;

    /* The panel is being hidden by SS_Frame */
    // hide();
    // setVisible(false);                 // for JDK 1.1
    if ((ontol != ont) || ontol == null)
      setCurrentOntology (ont);
	    // assume user did not reset ontology when it was changed...

    if (catname == null)
	    return;

    subjectCat = catname;

    removeAll();                            // remove all current components
    heading = new Label("Subject Category: " + subjectCat);
    Utilities.placeComponent(this, heading, 0, 0, 4, 1,
			     GridBagConstraints.EAST);

    /* get relations */
/* TEST
    System.out.println ("Ontol ID:  " + ont.getId());
    System.out.println ("Ontol version:  " + ont.getVersion());
    System.out.println ("Category name:  " + catname);
*/
    elmt_id = new Unique_Elt(ont.getId(), ont.getVersion(), catname);

    relList = ontman.getRelsByArgType(ont, elmt_id, 1);

    /* Get relations in the opposite direction. Note, it would be more
       efficient to find these at the same time as the other ones, since
       we now have to scan the ontology twice. If we had inferences, we
       could avoid this altogether by creating a relation for each inverse
       and then adding the appropriate inference rule to the ontology. */
    invRelsIdx = relList.size();
    Vector invRels = ontman.getRelsByArgType(ont, elmt_id, 2);
    Enumeration invEnum = invRels.elements();
    // add inverse relations to list
    while (invEnum.hasMoreElements()) {
      Object rel = invEnum.nextElement();
      relList.addElement(rel);
    }

    relCount = relList.size();
    relname = new String[relCount];
    l_fld = new Label[relCount];
    fldText = new TextField[relCount];
    bFind = new Button[relCount];
    cbShow = new Checkbox[relCount];

    /* for each relation update display */
    for (i=0; i < relCount; i++) {
      drel = (Def_Relation)relList.elementAt(i);
/* TEST
      System.out.println(i + "=" + drel.getName());
*/

      /* Note, in the case of RENAMEed objects, the above statement won't
	 assign the correct name from the perspective of the ontology. */

      if (i < invRelsIdx)
	relname[i] = drel.getName();
      else
	relname[i] = drel.getName() + " (INV)";
      // consider using drel.getLabel() above

      l_fld[i] = new Label(relname[i]);

      fldText[i] = new TextField (TEXTF_SIZE);
      fldText[i].setBackground(Color.white);
      bFind[i] = new Button (FIND_BUTTON_LABEL);
      cbShow[i] = new Checkbox (CHKBOX_BUTTON_LABEL);

      /* BUG: In Netscape 4 for Unix, the next four lines cause the
	 parent frame to drift to the upper left corner. */
      Utilities.placeComponent(this, l_fld[i], 0, i+1, 3, 1,
			       GridBagConstraints.EAST);
      Utilities.placeComponent(this, fldText[i], 3, i+1, 3, 1);
      Utilities.placeComponent(this, bFind[i], 6, i+1, 1, 1);
      Utilities.placeComponent(this, cbShow[i], 7, i+1, 1, 1);
    }

    /* The panel is being reshown by SS_Frame */
    // show();
    // setVisible(true);      // for JDK 1.1
    // validate();
  }

  /** Changes the default ontology for the query panel. */
  public void setCurrentOntology (Ontology newOnt)
  {
      ontol = newOnt;
  }


  /** Constructs and returns a keyword query using the values that have
    been supplied by the user in the property fields as well as the
    selected category. This is needed to implement the WebQueryString
    interface.
  */
  public String makeWebQuery()
  {
      StringBuffer  sBuf = new StringBuffer();

      //prevent null exception if main window has no QueryPanel displayed
      if (subjectCat == null)
          return  ("");

      /* Handle the category by first getting the source Def_Category object */
      Unique_Elt rootElmtId = null;
      try {
	rootElmtId = ontman.resolveName(subjectCat, ontol);
      } catch (SHOEException ex) {
	System.out.println("Couldn't find " + subjectCat + " in ontology.");
      }
      if (rootElmtId != null) {
	SHOE_Obj ob = ontman.getElement(rootElmtId);
	String prefTerm = ((Def_Category)ob).getLabel();
	sBuf.append ('"' + prefTerm.trim() + '"');
      }
      else
	sBuf.append (subjectCat);

      // For each non-empty textfield:
      //
      for (int i = 0; i < fldText.length; i++)
      {
         String  text = ((fldText[i]).getText()).trim();
         if (!text.equals(""))
         {
             boolean  hasa_space = spaceInText (text);

             Def_Relation drel = (Def_Relation)relList.elementAt(i);
             String  prefTerm = drel.getLabel();
             sBuf.append (" " + prefTerm.trim() + " ");

	     // prepend a '+' before the term to indicate that it's mandatory
	     sBuf.append("+");

	     // if multiple words, then quote them to indicate a phrase
             if (hasa_space)
	       sBuf.append ('"' + text + '"');
	     else
	       sBuf.append (text);

         }
      }
//TEST
//XXX
System.out.println ("Web query:  [" + sBuf.toString() + "]");

      return  (sBuf.toString());
  }

  boolean spaceInText (String  str)
  {
      if (str.indexOf(' ') >= 0)
         return  true;
      else
         return  false;
  }

  int  getRowSize()
  {
      return  relCount;
  }

  void disableRow (int index)
  {
      if ((index >= 0) &&(index < relCount))
      {
          (fldText[index]).setEditable (false);
          (cbShow[index]).disable();
      }
      // else, nothing happens
  }

  void enableRow (int index)
  {
      if ((index >= 0) &&(index < relCount))
      {
	  (fldText[index]).setEditable (true);
	  (cbShow[index]).enable();
      }
      // else, nothing happens
  }

  /** Puts given text in TextField for the relation indicated by the index.
     Indexing begins at 0.
  */
  void setFieldValue (int index, String textValue)
  {
      if ((index >= 0) && (index < relCount))
      {
	  (fldText[index]).setText (textValue);
	  (cbShow[index]).setState (true);  //on
      }
      else
      {
	  System.out.println (
                      "Tried to move text into a non-existant text field");
	  System.out.println ("   Ignored.  Field index is:  " + index);
      }
  }

/*  The following works in the JDK appletviewer, but not in netscape3.
    Investigation shows that action events are being generated for target
    Checkbox, and the correct match is occuring (see 'if'); however, the
    Checkbox does not seem to be enabled for focus events inside Netscape3,
    and, with 1.0 version java it is not possible to enable a component
    for events it does not act on.
*/
  public boolean action (Event evt, Object arg)
  {
/* TEST
      System.out.println ("ACTION EVENT");
      System.out.println ("  Target:  " + (evt.target).toString());
*/
      if (evt.target  instanceof  Checkbox)
      {
	  Checkbox  cb = (Checkbox)evt.target;
	  for (int i = 0; i < cbShow.length; i++)
	  {
/* TEST
              System.out.print ("  cb == cbShow[" + i + "]?  "); 
              System.out.println ( (cb == cbShow[i]));
*/
	      if (cb == cbShow[i])
	      {
		  cb.requestFocus();   //helps make UI look consistent;
                    //causes other buttons to turn on, if text just typed in
                    //to corr textfield
	          break;
	      }
	  }
	  return  true;
      }
      return  false;
  }

  public boolean lostFocus (Event evt, Object ob)
  {
      if (evt.target  instanceof  TextField)
      {
	  int  whichTextF = getTextFIndex ((TextField)evt.target);
	  String  text = ((fldText[whichTextF]).getText()).trim();
	  if (!text.equals(""))
          {
	      (cbShow[whichTextF]).setState (true);  //on
          }
	  return  true;
      }
      return  false;
  }

  /* Note, in the following function Event.ENTER, Event.BACK_SPACE, and
     Event.DELETE are constants from Java 1.1 */
  public boolean keyDown (Event evt, int key)
  {
      if (evt.id == Event.KEY_PRESS)
      {
	  if (evt.target  instanceof  TextField)
          {
	      // if text is added to a (perhaps empty field), make sure
	      // 'Show' is turned on automatically.
	      //
                 //do not return true for any of the following, or the
                 //TextField will not behave correctly. (Event still needs
		 //to propagate).
              if (evt.key == Event.ENTER)  // key == 13; same as Return key
	      {
		  int  whichTextF = getTextFIndex ((TextField)evt.target);
		  String  text = ((fldText[whichTextF]).getText()).trim();
		  if (text.equals(""))
		      (cbShow[whichTextF]).setState (false);  //off
			   //presumably some existing text was erased
		  else
		      (cbShow[whichTextF]).setState (true);  //on

		  return  false;
	      }
              else  if (evt.key == Event.BACK_SPACE)       // key == 8
	      {
		  int  whichTextF = getTextFIndex ((TextField)evt.target);
		  String  text = ((fldText[whichTextF]).getText()).trim();
		  if (text.length() == 1)  //then last char is being deleted:
		      (cbShow[whichTextF]).setState (false);  //off

		  return  false;
	      }
              else  if (evt.key == Event.DELETE)           // key == 127
	      {
		  int  whichTextF = getTextFIndex ((TextField)evt.target);
		  String  text = ((fldText[whichTextF]).getText()).trim();
		  String  selText = ((fldText[whichTextF]).
                                           getSelectedText()).trim();
		  if (text.equals (selText))
		      (cbShow[whichTextF]).setState (false);  //off

		  return  false;
	      }
          }
      }

      return false;
  }

  int getTextFIndex (TextField  findWhich)
  {
      int  which = -1;
      for (int i = 0; i < fldText.length; i++)
      {
	  if (findWhich == fldText[i])
	  {
	      which = i;
	      break;
	  }
      }
      return   which;
  }

  /** Returns index for 'find' button pressed.
  */
  int getFindButton (Button whichOne)
  {
      int  currButton = -1;
      int  i;
      for (i = 0; i < bFind.length; i++)
          if (bFind[i] == whichOne)
          {
              currButton = i;
              break;
	  }
      return  i;
  }

  /** Return a short, unique var name with some relationship to what
    it represents. 'i' is the position of the given attribute in the
    QueryPanel list, so that it is possible to determine where the
    variable was used simply by the variable name. */
  String makeVarName (String vName, int i)
  {
  /*  String  editedName = "?";
      if (vName.length() > VAR_TRUNC_SIZE)
         editedName = editedName + vName.substring (0, VAR_TRUNC_SIZE) + i;
      else
         editedName = editedName + vName + i;

      return  editedName;
   */
  /* NOTE:
     The variable names for Parka, and the names appearing on the user
     interface had been the same.  Now we are separating the two;  the
     internal rep for Parka will just be a letter and a number that matches
     the position of the given attribute in the QueryPanel list (starting
     at the top).  This way, the columns that appear in the ResultsWin can
     be matched to the corresponding attribute in the QueryPanel (first col ==
     first attribute, etc).  This is the assumption that you need to know
     to see how the single copy button works, that replaces a copy button
     associated with each line of the QueryPanel.
     Externally, we will see a different name generated for the column titles
     which come from the names of the QueryPanel rows (these are the relation
     names).
  */
     // The names for the parka variables will be constant length:
     // (makes finding the integer index easy)
     //
     return   (STD_PARKA_VAR_PREFIX + i);
  }

  /** Creates a new Parka variable to be used as the argPos'th argument
     of the relation indexed by relIndex. The relation index can be
     retrieved from the variable name using getVarRelIndex().
     */
  public String makeVarName(int relIndex, int argPos)
  {
    String varName;

    /*
    if (argPos < 3)
      varName = "?" + "X" + relIndex;
    else
    */
      varName = "?" + "X" + relIndex + "_" + argPos;
    return varName;
  }

  /** This function creates a single conjunct that uses the predicate
    corresponding to the Def_Relation. The first two arguments in the
    query correspond to the arguments passed to the function; if the
    relation requires other arguments, new variables are created for
    them. Note, if the index corresponds to an inverse relation then
    the arguments are reversed.
    */
  public Atom makeRelQuery(int relIndex, String arg1, String arg2)
  {
    Def_Relation drel;
    int          arity;
    String[]     args;
    String       var;
    Atom         query;
    int i;

    drel = (Def_Relation)relList.elementAt(relIndex);
    arity = drel.getArity();
    args = new String[arity];

    if (relIndex < invRelsIdx) {
      args[0] = arg1;
      args[1] = arg2;
    }
    else {
      args[0] = arg2;
      args[1] = arg1;
    }

    if (arity > 2)
    {
       for (i=3; i <= arity; i++) {
          var = makeVarName(relIndex, i);
	  args[i - 1] = var;
      }
    }

    /* BUG: If we use the rel's name, we do not properly specify the query
            in cases where the current context imported the relation with
	    a new name! i.e., the name can't be resolved in the context */
    query = new Atom(drel.getName(), args);
    return query;
  }

  /** Create a Parka query string that finds all relations where the
      predicate is of the given index.
  */
  public Query getFindQuery(int whichFindButton)
  {
     int          argPos;           // argument position of related object 
     String       tVar;
     String       qCat;
     Atom         relQry;
     Query        query;

     // determine which position the object occupies in the relation
     if (whichFindButton < invRelsIdx)
       argPos = 2;
     else
       argPos = 1;

     tVar = makeVarName (whichFindButton, argPos);
     qCat = STD_PARKA_KEY_VAR;
     query = new Query(ShoeKb.P_EVERY_INSTANCE, subjectCat, qCat);
     relQry = makeRelQuery (whichFindButton, qCat, tVar);
     query.addConjunct(relQry);

     return query;
  }

  /** Create a Parka query string that finds all relations where the
    predicate is of the given index. Unlike getFindQueryString, this
    returns the names as well as the keys of the related objects
  */
  public Query getFindNamesQuery(int whichFindButton)
  {
     int          argPos;         // argument position of related object
     String       tVar;
     String       qCat;
     Query        query;
     Atom         relQry;

     // determine which position the object occupies in the relation
     if (whichFindButton < invRelsIdx)
       argPos = 2;
     else
       argPos = 1;

     tVar = makeVarName (whichFindButton, argPos);
     qCat = STD_PARKA_KEY_VAR;
     query = new Query(ShoeKb.P_EVERY_INSTANCE, subjectCat, qCat);

     // find name corresponding to the key
     query.addConjunct("name", qCat, STD_PARKA_KEYNAME_VAR);
     relQry = makeRelQuery(whichFindButton, qCat, tVar);
     query.addConjunct(relQry);

     /* if the related object is a key, find the name of the related object */
     Def_Relation drel = (Def_Relation) relList.elementAt(whichFindButton);
     if (drel.getArity() >= 2) {         // all rels should be
       Def_Arg[]  argList = drel.getDefArgs();
       int typeClass = (argList[argPos - 1]).getTypeClass();
           //index is one less than argument position

       if (typeClass == Def_Arg.CATEGORY) {
          String  relNameVar = STD_PARKA_NAMEVAR_PREFIX + whichFindButton;
	  query.addConjunct("name", tVar, relNameVar);
       }
     }

     return query;
  }

  /*
    // we'll use the isDigit from java.lang.Character instead
  boolean isDigit (char c)
  {
      switch (c) //we will not depend on any particular internal rep:
      {
         case '0': case '1': case '2': case '3': case '4':
         case '5': case '6': case '7': case '8': case '9':
                   return  true;
         default:
                   return  false;
      }
  }
  */

  /** Returns true if text is numeric or false if it is not. */
  /* We use this methods instead of throwing a number format
     exception because we would rather not throw exceptions all the 
     time for a situation that is common; we use this routine instead. 
     It should really be a static in the Utilities class.  */
  /* BUG: I don't think this allows decimal values */
  boolean isNumeric (String text)
  {
      boolean  numeric = true;

      for (int i = 0; i < text.length(); i++)
      {
	if (!Character.isDigit (text.charAt(i)))
         {
            numeric = false;
            break;
         }
     }
     // TEST
     // System.out.println (numeric + " --is numeric: " + text);

      return  numeric;
  }

  /** Returns a Query object that corresponds to the user's inputs.
    The query only includes items if Show button is on, or if text has
    been entered to select data. Note that the makeRelQuery() routine
    handles ordering inverse relations properly. However, we have to make
    sure the right argument position is used when creating the variable,
    so that a correct description can later be derived from the variable.
  */
  public Query getQuery()
  {
     int          argPos;         // argument position of related object
     String       text;
     Query        query;
     Atom         relQry;

     String   qCat = STD_PARKA_KEY_VAR;

     query = new Query(ShoeKb.P_EVERY_INSTANCE, subjectCat, qCat);

     for (int i=0; i < relname.length; i++)
     {
        text = fldText[i].getText();
        text.trim();   //elim blanks from both ends

	// determine which position the object occupies in the relation
	if (i < invRelsIdx)
	  argPos = 2;
	else
	  argPos = 1;

        //only include item in output if Show button is on, or
        //if text is used for selecting data
        //
        //first deal with fields with text in them
        //

        if (!text.equals(""))
        {
           Def_Relation drel = (Def_Relation) relList.elementAt(i);
           if (drel.getArity() >= 2)
           {         // all rels should be
              Def_Arg[]  argList = drel.getDefArgs();
              int typeClass = (argList[argPos - 1]).getTypeClass();
                                   //indices are one less the agument positions

              if (typeClass == Def_Arg.CATEGORY)
              {
                 if (text.startsWith(URL_PREFIX))
                 {
                    relQry = makeRelQuery(i, qCat, text);
		    query.addConjunct(relQry);
                 } 
                 else
                 {
                    // if not a URL, use string match on the "name" relation
                    String  field = makeVarName (i, argPos);
                    relQry = makeRelQuery(i, qCat, field);
		    query.addConjunct(relQry);
                    /* Part of separating Parka variable names from external
                      names in user interface; see comment in 'makeVarName()'. 
                    */
                    String  strPart = STD_PARKA_NAMEVAR_PREFIX + i;
		    query.addConjunct("name", field, strPart);
		    query.addConjunct(ShoeKb.P_NOCASE_STRING_MATCH, strPart,
				      "%" + text + "%");
                 }
              }
              /* a type check is done on the entries for numeric fields. Also,
                 for now, numeric text must exactly match; later, perhaps
                 we should incorporate ability to do exact or ranges.
                 (Range is a Parka built-in).
                 */
              else if (typeClass == Def_Arg.NUMBER)
              {
                 if (isNumeric(text))    //then type is OK
                 {
                    relQry = makeRelQuery(i, qCat, text);
		    query.addConjunct(relQry);
/* TEST
 	            System.out.println (text + "  is a number.  Type chk, OK");
*/
                 }
                 else
                    System.out.println ("Non-numeric value in numeric field");
              }
              /* perform a string match on arguments of any other type
                 only numeric type check is done currently; should eventually
                type check any relation. */
              else
              {
                 String  field = makeVarName (i, argPos);
                 relQry = makeRelQuery(i, qCat, field);
		 query.addConjunct(relQry);
		 query.addConjunct(ShoeKb.P_NOCASE_STRING_MATCH, field,
				   "%" + text + "%");
              }
           }
           else
              System.out.println ("Don't have at least two arguments defined for this relation");
        }

        // add var if field is empty but "Show" is checked 
        else if (cbShow[i].getState())
        {       
           String tVar = makeVarName (i, argPos);
           relQry = makeRelQuery(i, qCat, tVar);
	   query.addConjunct(relQry);
        }
      }        /* end 'for' */    

     return query;
  }

  /** Given a PARKA variable name, retrieves the part that represents
    the relation index. Note that there are 3 types of variables:
    ?K:    the key of an instance
    ?KN:   the name corresponding to the key
    ?Xr_a: a variable in a relation r, argument position a
    ?Nr:   the name corresponding to to the 2nd argument in relation r
  */
  public int getVarRelIndex(String varname)
  {
    int rnum = 0;
    int sep;
    String idxStr;

    if (varname.startsWith("?X")) {
      sep = varname.indexOf('_');
      idxStr = varname.substring(2, sep);
    }
    else if (varname.startsWith("?N"))
      idxStr = varname.substring(2);
    else
      return -1;
    try {
      rnum = Integer.parseInt(idxStr);
    } catch (Exception ex) {
      System.out.println(varname + " is of incorrect format");
      System.out.println(idxStr + " is not a number");
    }
    return rnum;
  }

  /** Given a PARKA variable name, retrieves the part the represents
    the argument position */
  public int getVarArgPos(String varname) {

    int argPos = -1;
    int sep;
    String numStr;

    if (varname.startsWith("?X")) {
      sep = varname.indexOf('_');
      numStr = varname.substring(sep + 1);
      try {
        argPos = Integer.parseInt(numStr);
      } catch (Exception ex) {
        System.out.println(varname + " is of incorrect format");
        System.out.println(numStr + " is not a number");
      }
    }
    else
      argPos = -1;
    return argPos;
  }

  /** Given an array of variable names, returns an array of the
    corresponding English descriptions
  */
  public String[] getVariableDescs(String varnames[])
  {
    int i;
    int rnum = 0;
    int argPos;
    String desc;
    Def_Relation drel;
    Def_Arg darg;

    String descList[] = new String[varnames.length];
    for (i=0; i < descList.length; i++)
    {
      rnum = getVarRelIndex(varnames[i]);
      if (rnum >= 0)
      {
         if (varnames[i].startsWith("?X"))
         {
            desc = relname[rnum];
            argPos = getVarArgPos(varnames[i]);

            /* for relations with more than 2 args, identify the arg */
            drel = (Def_Relation)relList.elementAt(rnum);
            if (drel.getArity() > 2) {
               darg = drel.getDefArg(argPos);
               desc = desc + " (" + darg.getLabel() + ")";
           }
         }
         else
            desc = relname[rnum] + "(name)";
      }
      else
      {
        desc = subjectCat;
        if (varnames[i].equals(STD_PARKA_KEYNAME_VAR))
           desc = desc + "(name)";
      }
      descList[i] = desc;
    }

    return descList;
  }

}

/** This class is use to store information on a given Parka variable */
/*
class Variable extends Object {

  public String  name;
  public int     relIndex;
  public int     argPos;
  public String  desc;

  public Variable(String name, int relIndex, int argPos, String desc) {

    this.name = name;
    this.relIndex = relIndex;
    this.argPos = argPos;
    this.desc = desc;
  }
}
*/
