/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

package  ShoeSearch;

import ShoeKb.*;
import UserInt.MultiColumn;
import UserInt.MultiColumnCallback;
import java.awt.*;
import java.util.*;
import java.applet.*;
import java.net.*;

/* NOTE:  THIS IS A VERSION OF ResultsWin THAT REQUIRES AN APPLET CONTEXT
          TO WORK.
	  Other changes from the versions in the Parka Client Apps:
	1. No 'doQuery()' routine.  'changeTable()' is very similar,
           altho the query has been done by this point.  Query is done
	   from SS_Frame class.
	2. No 'createQueryString()'.  Replaced with
           'QueryPanel.getQueryString()'.
	3. No need for assertions (in nary parka client only).
	   'createAssertionString()' removed.
	4. singleClick() and doubleClick() do different things.
*/

/** A class that manages the results of a knowledge base query. */
public class ResultsWin  extends Panel  implements MultiColumnCallback
{
    Frame         parentWin;
    MultiColumn   resultList;
    ResultSet     rset;
    AppletContext context;    // used to load documents in web-browser

    static private String   START_LBL_STR = "First Item: ";
    static private String   END_LBL_STR = "Last Item: ";
    static private int      NUMCOLS_TEXTF = 10;
    static private int      GRIDROWS = 1;
    static private int      GRIDCOLS = 3;
    static private int      HORGAP = 2;
    static private int      VERTGAP = 2;

    Panel         outputIndicators;
    TextField     startText;
    TextField     endText;
    Button        bNext;
    Button        bPrev;

    int           totalNumAnswers;
    int           numCols;
    int           firstMultiListItem;
    int           lastMultiListItem;
    static private int    blockSize; //number of answers obtained per call
                                     //to server

    static final String   NOVARSINQUERY = "No variables retrieved";
         //MUST match value returned from server when request for
         //variables is given.

    ResultsWin (Frame parWin, AppletContext context)
    {
        super();

        parentWin = parWin;
        rset = null;
	// this.kb = kb;
        this.context = context;

        resultList = null;

        blockSize = 100;      //arbitrary choice, but small enough not
                              //to upset Vectors inside MultiColumn.
//TEST
//	blockSize = 6;

        setupIndicatorPanel();
        initIndicators();          //do after 'setupIndicatorPanel()'
        setLayout (new BorderLayout (0, 0));
        add("North", outputIndicators);

        //create a single empty MultiColumn, to start.
        emptyMultiColumn();
        validate();
    }


  /** Replace the old result list with a new, empty one. */
    public void newEmptyMultiColumn()
    {
      initIndicators();
      remove(resultList);         // remove old resultList
      emptyMultiColumn();         // replace with one column empty list
      // resultList.clear();
      // resultList.erase_titles();
    }

    public ResultSet getQR()
    {
        return  rset;
    }

    public void setQR (ResultSet qRes)
    {
	     rset = qRes;
    }

    public MultiColumn  getMultiColumn()
    {
	return  resultList;
    }

    public void clear()
    {
	initIndicators();
	if (resultList != null)
	{
	    resultList.clear();
	    resultList.erase_titles();
	}
    }

    public boolean action (Event e, Object ob)
    {
      if (e.target  instanceof  Button)
	{
	  if (e.target == bNext)
	    {
	      showNextGroup();
	      return   true;
	    }
	  else if (e.target == bPrev)
	    {
	      showPreviousGroup();
	      return   true;
	    }
	  else
	    System.out.println ("Programmer Error -- " +
				"Cannot recognize button used");
	}
      return   false;
    }

  /*
    // No longer used
    int getNumResults()
    {
	if ((rset != null) && (rset.getNumResults() > 0))
	    return  (rset.getNumResults());
	else
	    return  0;
    }
    */

  /** Updates the result list table to use the column headings given
    by heading and the query results given by rset. The table is
    updated to display the first group of answers. */
    public void changeTable (String headings[], ResultSet rset)
    {
      setQR(rset);                     // update the local results variable

      if (resultList != null)
        remove(resultList);            // remove the old resultList
      resultList = null;
  
      if (rset == null)
        {
	  changeMessage ("Error issuing query.");
	  newEmptyMultiColumn();             // also resets global variables
        }
      else
        {
	  if (rset.getNumResults() == 1)
            {
	      changeMessage("1 answer found.");
            }
	  else {
	    changeMessage(rset.getNumResults() + " answers found.");
	  }
	  
	  resultList = new MultiColumn (headings, this);
 /*
            System.out.println ("MultiList with ResultsWin as callback");
*/
	  add("Center", resultList);


	  totalNumAnswers = rset.getNumResults();
	  if (totalNumAnswers > 0)
            numCols = rset.getNumColumns();  //var names
	  else
	    numCols = 0;

	  showFirstGroup();
        }
      validate();
    }

  /** Called on a single click on a list item.
       The selected list items are: 0 for the first item,
       1 for the second, etc, thru n-1 for the nth.
    */
    public void singleClick (MultiColumn list, int row, int col)
    {
	// TEST
	// System.out.println ("Single click -- Row, Col: " + row + col);
    }


    /* Heflin 5/8:  Changed this so that applets would load a document
       into the browser based on the user's click. */
  /** Called upon double-clicking on a list item.
    */
    public void doubleClick(MultiColumn list, int row, int col)
    {
        /*
        // Use double click to clear the effect of single clicking
        // on the list.
	//
        singleClick (list, -1);
	*/

	// TEST
	// System.out.println ("Double click -- Row, Col: " + row + col);

        URL url = null;

        if (context != null) {                  // only works for applets
 	  /* retrieve bindings  */
	  String[] binds = list.getItem(row);
	  if (binds[col].startsWith (SS_Frame.URL_PREFIX)) { // probably a URL
	    try {
	      url = new URL(binds[col]);
	      System.out.println("Trying to show " + url);
	      // display document in a new window named "SHOE_Doc"
	      context.showDocument(url, SS_Frame.WEBPG_TARGET);
	    }
            catch (MalformedURLException ex) {
	      System.out.println(url + " not valid"); 
	    }
	  }
        }
    }


  /** Displays the initial group of answers in the table. */
  public void showFirstGroup()
  {
    if (totalNumAnswers > 0)
      firstMultiListItem = 1;
    else
      firstMultiListItem = 0;
    lastMultiListItem = Math.min (blockSize, totalNumAnswers);

    if (bPrev.isEnabled())
      bPrev.disable();

    if (lastMultiListItem < totalNumAnswers)
      bNext.enable();
    else if (bNext.isEnabled())
      bNext.disable();

    setTextFields();
    putAnswersInMultiList();
    validate();
    repaint();
  }


  /** Displays the next group of answers in the table. */
  public void showNextGroup()
  {
    firstMultiListItem += blockSize;
    lastMultiListItem += blockSize;
    if (lastMultiListItem >= totalNumAnswers)
      {
	lastMultiListItem = totalNumAnswers;
	bNext.disable();
      }
    setTextFields ();
    if (!bPrev.isEnabled())
      bPrev.enable();
    putAnswersInMultiList();
    validate();
    repaint();
  }

  /** Displays the first group of answers in the table. */
  public void showPreviousGroup()
  {
    lastMultiListItem = firstMultiListItem - 1;
    firstMultiListItem = (lastMultiListItem - blockSize) + 1;

    if (firstMultiListItem <= 1)
      {
	firstMultiListItem = 1;
	bPrev.disable();
      }
    //readjust, to make sure we have a whole blocksize of answers:
    if (firstMultiListItem == 1)
      lastMultiListItem = blockSize;

    setTextFields ();
    if (!bNext.isEnabled())
      bNext.enable();
    putAnswersInMultiList();
    //not tested; using the one requiring ResultSet.
    validate();
    repaint();
  }


    void changeMessage (String msg)
    {
        String statusMsg = msg;
        //  System.out.println ("ResultsWin status = " + statusMsg);
    }


  /* ******* Private Methods ******** */

  /** Create a single empty MultiColumn. */
    private void emptyMultiColumn()
    {
        String[] initlist = new String[1];
        initlist[0] = " ";
/*
        System.out.println ("MultiColumn without a callback");
*/
        resultList = new MultiColumn (initlist);
		  //single empty column header
        add("Center", resultList);
    }


    private void setupIndicatorPanel()
    {
	outputIndicators = new Panel();
	outputIndicators.setLayout (new GridLayout(GRIDROWS, GRIDCOLS,
                        HORGAP, VERTGAP));
	Label  start = new Label (START_LBL_STR, Label.LEFT);
	Label  end = new Label (END_LBL_STR, Label.LEFT);
	startText = new TextField (NUMCOLS_TEXTF);
	startText.setBackground(Color.lightGray);
	startText.setEditable (false);
	endText = new TextField (NUMCOLS_TEXTF);
	endText.setBackground(Color.lightGray);
	endText.setEditable (false);
	Panel  txtIndStart = new Panel();  //FlowLayout is default
	txtIndStart.add (start);
	txtIndStart.add (startText);
	Panel  txtIndEnd = new Panel();    //FlowLayout is default
	txtIndEnd.add (end);
	txtIndEnd.add (endText);

	Panel buttonPanel = new Panel();
	bNext = new Button("  Next Group  ");
	bPrev = new Button("Previous Group");
	buttonPanel.add(bPrev);
	buttonPanel.add(bNext);
        outputIndicators.add (txtIndStart);
        outputIndicators.add (txtIndEnd);
        outputIndicators.add (buttonPanel);
    }


    private void initIndicators()
    {
        firstMultiListItem = 0;
        lastMultiListItem = 0;
        numCols = 0;
        totalNumAnswers = 0;
        setTextFields();
        bPrev.disable();  //must have already created.
        bNext.disable();  //must have already created.
    }


    private void setTextFields()
    {
	startText.setText (String.valueOf(firstMultiListItem));
	endText.setText (String.valueOf(lastMultiListItem));
    }



  /** Loads items from firstMultiListItem to lastMultiListItem of the
    current result set into the MultiList. Must be called sometime
    after query is submitted to server, and after firstMultiListItem
    and lastMultiListItem are correctly set. */
    private  void putAnswersInMultiList()
    {
        // If we do not call 'MultiColumn.clear()', answers just
        // accumulate.  Of course that would require different handling
        // than we do here, since when a user would go back and forward
        // multiple times with the current way of doing things, we'd
        // just accumulate more and more answers (same ones appending).
        //

      resultList.clear();	
      if (firstMultiListItem > 0 && lastMultiListItem >= firstMultiListItem) {
	
        int numRows = Math.min (blockSize, (lastMultiListItem -
                                   firstMultiListItem) + 1);
        int  start = firstMultiListItem -1;
        int  end   = start + numRows;
/* TEST
        System.out.println ("numRows == " + numRows);
        System.out.println ("start == " + start);
        System.out.println ("end == " + end);
*/
        for (int i = start; i < end; i++)
        {
            resultList.addItem(rset.getResult(i));
        }
      }
    }

}

