/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

package  ShoeSearch;

import java.util.*;
import java.io.*;

public class ThesResponse extends java.lang.Object
{
    protected  int      status = 0; // ok
    protected  int      size;
    protected  int      numAnswers;
    protected  String   response;
    protected  InputStream instream;

	 //for delimiting tokens.
    protected  static final  String  TAB_DELIM = "\t";
    protected  static final  String  SPACE_DELIM = " ";
    protected  static final  String  VERTLINE_DELIM = "|";
    protected  String[][]  tokenList;

    final static int     NUMTERMTYPES = 3;
    final static int     XCEPTION = -10;
	 //when this kind of error happens, might want to use the
	 //'response' field to display to ultimate user.

    public ThesResponse (InputStream in)
    {
	    size = 0;      //length of response string from server
	    response = "";
	    // init query related member variables:
	    tokenList = null;
	    numAnswers = 0;
	    instream = in;  //input stream assoc. with socket
    }

    /* The format for the thesaurus server is:
           1. size of response, then blank space
           2. number of responses, then blank space
	        3. the response
	     The format within the reponse depends on the type of thesaurus
        answer.
    */
    void computeResponse()
    {
        // System.out.println ("Entered ThesResponse.computeResponse()");

        try
        {
            StringBuffer in = new StringBuffer();
            int tmp;
            boolean exception_occured = false;

            //determine if hung waiting for server to respond:
/*          System.out.println (
                         "Before 'instream.read()' for number of answers");
*/
	         // Get number of incoming answers
            tmp = instream.read();
/*
            System.out.println (
                         "After 'instream.read()' for number of answers");
*/
            while ((tmp != -1) && (((char)tmp) != ' '))
            {
                in.append((char)tmp);
                tmp = instream.read();
		          // System.out.print((char)tmp);
            }
	         // System.out.println();

            try
            {
                numAnswers = Integer.parseInt(in.toString());
            }
            catch (NumberFormatException e)
            {
                System.out.println("Error parsing numAnswers: [" +
                                        in.toString() + "]");
		          exception_occured = true;
            }
/* TEST
	         System.out.println("ThesResponse:  Number of answers = "
                                    + numAnswers);
*/
            // Get size;  read until first space
            in = new StringBuffer();

            tmp = instream.read();
            while ((tmp != -1) && (((char)tmp) != ' '))
            {
                in.append((char)tmp);
                tmp = instream.read();
            }

            try
            {
                size = Integer.parseInt(in.toString());
            }
            catch (NumberFormatException e)
            {
                System.out.println("Error parsing size: [" +
                                      in.toString() + "]");
		          exception_occured = true;
            }
/* TEST
	         System.out.println("ThesResponse:  Size = " + size);
*/
	         if (!exception_occured)
	         {
                // Get response
                byte[] bytearr = new byte[size];
                int readsize = 0;
                //System.out.println("Reading");
                while (readsize < size)
                {
                    readsize += instream.read (bytearr, readsize,
                                                (size-readsize));
                }
                response = new String(bytearr, 0);
            }
	         else
	         {
		          response = "Communication failure with Server";
		          size = response.length();
		          status = XCEPTION;
	         }

            // output will be:
/* TEST
            System.out.println ("size = "
                              + size + "; numAnswers = " + numAnswers +
                              "; response = " + response);
*/
        }
        catch (IOException e1)
        {
            System.out.println ("IO Error in getResponse()");
            status  = XCEPTION;
            response = "Communication failure with Server";
            size    =  response.length();
        }
    }

    // if ThesResponse is for the EXPAND command
    // Answer back has a tokenList[i][] for each thesaurus with > 0 answers
    //   in it.  (7 items from each thesaurus).
    //
    protected void makeDataTokens()
    {
        StringTokenizer  responseTokens = new StringTokenizer (
                                      response, TAB_DELIM);
        tokenList = new String[numAnswers][];

        for (int i = 0; i < numAnswers; i++)
        {
            StringTokenizer  subStringTokens = new StringTokenizer (
                                  responseTokens.nextToken(), VERTLINE_DELIM);
            String  firstToken = "", littleTokenString = "";

				if (subStringTokens.countTokens() == 2)
            {
					 firstToken = subStringTokens.nextToken();
                littleTokenString = subStringTokens.nextToken();
            }
            else
                System.out.println ("Warning!  Should get only two tokens here!");

            StringTokenizer  littleTokens = new StringTokenizer (
                                    littleTokenString, SPACE_DELIM);

            String[]   answerList = new String[1 + littleTokens.countTokens()];

            answerList[0] = firstToken;
            int  j = 1;
            while (littleTokens.hasMoreTokens())
            {
                answerList[j] = littleTokens.nextToken();
/* TEST
                System.out.println ("   Next TOKEN:  " + answerList[j]);
*/
                j++;
            }
            tokenList[i] = answerList;
         }
    }

    // if ThesResponse is for the GETRESONSE command
    // Answer back is: tokenList[0][] are the broader terms;
    //                 tokenList[1][] are the narrower terms;
    //                 tokenList[2][] are the related terms;
    //
    protected void makeTermsTokens()
    {
        StringTokenizer  responseTokens = new StringTokenizer (
                                                  response, TAB_DELIM);
        tokenList = new String[NUMTERMTYPES][];

        for (int i = 0; i < NUMTERMTYPES; i++)
        {
            String  numStr;
            try  {
                numStr = responseTokens.nextToken();
            }
            catch  (NoSuchElementException  noneE)  {
                numStr = null;
                System.out.println ("No next token exists");
           }
           int   num;
           try
           {
               num = Integer.parseInt(numStr);
           }
           catch (NumberFormatException e)
           {
               status = XCEPTION;
               break;
           }
           String  term[] = new String[num];
           for (int j = 0; j < num; j++)
           {
               try  {
               term[j] = responseTokens.nextToken();
               }
               catch  (NoSuchElementException  noneE)  {
                   term[j] = null;
                   System.out.println ("No next token exists");
               }
           }
           tokenList[i] = term;
       }
   }

    public int getStatus()
    {
        return  status;
    }

    public int getSize()
    {
        return  size;
    }

    public int getNumAnswers()
    {
        return  numAnswers;
    }

    public String getResponse()
    {
        return  response;
    }

    public String[][] getTokenList()
    {
        return   tokenList;
    }

    //for debug:
    void printTokenList()
    {
        System.out.println ("\nThesResponse.tokenList[][]: DIM 1-- " +
                            tokenList.length);
        for (int i = 0; i < tokenList.length; i++)
        {
            if (tokenList[i] != null)
            {
                System.out.println ("   tokenList[][]: DIM 2-- " +
                                        tokenList[i].length);
                for (int j = 0; j < tokenList[i].length; j++)
	                 System.out.print (" " + tokenList[i][j]);
                System.out.println("");
            }
            else
	             System.out.println ("tokenList[" + i + "] is null");
        }
        System.out.println ("");
    }
}
