/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

package  ShoeSearch;

import  java.awt.*;
import  java.applet.*;
import  java.net.*;
import  UserInt.ApplType;
import  UserInt.GridBagCreator;

/** A dialog window that allows the user send a variation of the
  SHOE Search query to any one of a number of search engines. */
public class  WebSearch  extends Dialog
                         implements  WebSearchHelpContainer
{
    WebSearchContainer parent;
    AppletContext   context;
    TextArea        webQuery;        // text field for search terms
    Button          bCopy;
    Button          bExec;
    Button          bThesaurus;
    Choice          searchEngine;
    Panel           typeSearchCont;
    Button          bQuit;
    WebQueryString  qStrGenerator;
    WebQHelp        doWebHelp;   //a query helper window with thesauri

    static  final  String  QUIT_BUTTON_LABEL = "Quit";
    static  final  String  EXEC_BUTTON_LABEL = "Execute";
    static  final  String  THES_BUTTON_LABEL = "Query Help";
    static  final  String  COPY_BUTTON_LABEL = "Copy Query";
    static  final  String  QUERY_LABEL = "Your Query Is:";
    static  final  String  CHOICE_LABEL = "Search Engine:  ";

  /** when true, this parameter will use phrase quoting if it is
    applicable to the selected search engine. */
    static  final boolean  QUOTE_PHRASES = true;


  /* information on search engines used by this class. The first column
     is the name, the second is the URL (including cgi script name and
     required parameters, and the third indicates if quotes can be used
     to indicate phrases (Yes or No). */
    static final String[][] engines =
    {{"AltaVista", "http://www.altavista.com/cgi-bin/query?pg=q&kl=XX&q=", "Y"},
     {"Ask Jeeves", "http://www.askjeeves.com/main/askjeeves.asp?ask=", "Y"},
     {"Excite", "http://search.excite.com/search.gw?search=", "Y"},
     {"FAST", "http://www.alltheweb.com/cgi-bin/asearch?type=any&query=", "Y"},
     {"Go / Infoseek", "http://www.infoseek.com/Titles?qt=", "Y"}, 
     {"Go2Net / MetaCrawler", "http://search.metacrawler.com/crawler?method=1&general=", "Y"},
     {"Google", "http://www.google.com/search?q=", "Y"},
     {"LookSmart", "http://www.looksmart.com/r_search?l&key=", "Y"},
     {"Lycos", "http://www.lycos.com/cgi-bin/pursuit?query=", "N"},
     {"Northern Light", "http://www.northernlight.com/nlquery.fcg?cb=0&qr=", "Y"},
     // {"PubMed", "http://www.ncbi.nlm.nih.gov/htbin-post/Entrez/query?form=4&db=m&dispmax=20&relentrezdate=No+Limit&term=", "N"},
     {"Yahoo", "http://ink.yahoo.com/bin/query?p=", "Y"}
    };


    /* In general, the search engines have additional parameters added after
       the parsed phrase that the search is done on, which you see if you
       type in the search directly for the given search engine.  However,
       the searches seem to work without these parameters (some of which
       seem to be added by netscape to tell the search engine what kind of
       displays it can handle; there may be other purposes as well).
    */

  /* 

     Alta Vista: Does the regular query, not the advanced one.  From notes
          on AltaVista's help page, the regular form looks better for this,
	  anyway. NOTE for later:  altavista has "fancy key word features",
	  e.g., can search for words specifically in URLs.
     Excite: inside netscape, the following is appended, although the search
             appears to work ok without it:   &sorig=netscape
     Lycos: treats quotes literally, not as phrases. Also performs an 'and'
            search as its default rather than 'or', like many other search
	    engines do. The following can be inserted after the '?' and
	    before the 'query':    matchmode=and&cat=lycos&
            but it seems to make no difference to the search results (on a
            very limited number of tries).

    */

    static  final  String  WEB_TARGET = "Web Search Result";

    public WebSearch (WebSearchContainer parWin, String title,
		      AppletContext ac, WebQueryString  qStrGenerator)
    {
       super ((Frame)parWin, title, false);
          //want user to be able to change items in the parent
          //window, therefore we choose not to use a modal dialog.
       parent = parWin;
       context = ac;
       this.qStrGenerator = qStrGenerator;
       doWebHelp = null;

       createDisplay();

       /* set default value of query to be search engine version of the
          QueryPanel query */
       String  webQStr = qStrGenerator.makeWebQuery();
       webQuery.setText (webQStr);

       if (ApplType.getIsApplication())
       {
           System.out.print ("This application cannot display web search ");
           System.out.print ("results unless ");
           System.out.println ("run as an Applet.");
       }
    }

    private void createDisplay()
    {
       GridBagCreator   gbcreat = new GridBagCreator (this);

       Label  qLabel = new Label (QUERY_LABEL, Label.LEFT);
       bCopy = new Button (COPY_BUTTON_LABEL);
       webQuery = new TextArea ("", 2, 80); //can't control scrollbars in 1.0.
          //text area init'd with 2 rows, 80 cols, blank text
       webQuery.setBackground(Color.white);
       bExec = new Button (EXEC_BUTTON_LABEL);
       bThesaurus = new Button (THES_BUTTON_LABEL);
       bQuit = new Button (QUIT_BUTTON_LABEL);
       setupChoice();

       gbcreat.setAnchor (GridBagConstraints.WEST);
       gbcreat.addComponent (qLabel,
                                  0,  0, 3, 1,
                                  80,  0,  GridBagConstraints.NONE, 3, 2);
       gbcreat.setAnchor (GridBagConstraints.EAST);
       gbcreat.addComponent (bCopy,
                                  3,  0, 1, 1,
                                  20,  0,  GridBagConstraints.NONE, 3, 2);
       gbcreat.setAnchor (GridBagConstraints.WEST);
       gbcreat.addComponent (webQuery,
                                  0,  1, GridBagConstraints.REMAINDER, 1,
                                  0,  100, GridBagConstraints.BOTH, 3, 2);
       gbcreat.addComponent (typeSearchCont,
                                  0,  2, 1,  1,
                                  40, 0, GridBagConstraints.NONE, 3, 2);
       gbcreat.setAnchor (GridBagConstraints.CENTER);
       gbcreat.addComponent (bExec,
                                  1,  2, 1,  1,
                                  20, 0, GridBagConstraints.NONE, 3, 2);
       gbcreat.addComponent (bThesaurus,
                                  2,  2, 1,  1,
                                  20, 0, GridBagConstraints.NONE, 2, 2);
       gbcreat.addComponent (bQuit,
                                  3,  2, 1,  1,
                                  20, 0, GridBagConstraints.NONE, 2, 2);
       pack();
       validate();
    }

    private void setupChoice()
    {
       int i;

       typeSearchCont = new Panel();
       typeSearchCont.setLayout (new GridLayout (1, 2));

       Label   choiceLabel = new Label (CHOICE_LABEL, Label.LEFT);
       searchEngine = new Choice();
          //first item added will be the default item, initially.

       for (i=0; i < engines.length; i++) {
	 searchEngine.addItem (engines[i][0]);
       }

       typeSearchCont.add (choiceLabel);
       typeSearchCont.add (searchEngine);
    }

    // Creates the URL with the correctly formatted query string embedded
    // for the currently chosen search engine.
    //
    String getWebQuery()
    {
       int  selection = searchEngine.getSelectedIndex();
       String  urlWithQuery;

       urlWithQuery = makeSearchQuery(engines[selection][1],
				      engines[selection][2]);

       //empty string may be returned, if problem found during processing

       return  urlWithQuery;
    }





  /** Given a query URL and a "Y/N" indicator of whether it accepts 
     quotation marks as phrase indicators, returns a URL that queries
     the search engine using the text entered by the user. */
    String makeSearchQuery(String loc, String phrases)
    {
        StringBuffer  queryStr = new StringBuffer();
        String  webQ;

        queryStr.append (loc);
        String  searchWords = webQuery.getText();   // get search terms

	/* if phrases are not recognized, replace double-quotes with spaces */
	if (QUOTE_PHRASES == false || phrases.equals("N")) 
	  searchWords = searchWords.replace('"',' ');

	if (searchWords != null) {
	  webQ = URLEncoder.encode(searchWords);  // replaces queryParse()
	  /* URLEncoder is a standard class that translates a string to
	     a format suitable for use in a URL. Spaces are converted to
	     plus signs("+") and quotes to "%22". Thus: '"word1 word2" word3'
	     becomes %22word1+word2%22+word3. Additional notes: the '+' that
	     means "required word" is coded  %2B, a '-' meaning 
	     "required not-to-have word" is just -. The user would have to
	     edit the text in the Web Search window to get the pluses
	     &/or minuses.
	   */
	  queryStr.append (webQ);
	}
        else
	  return  ("");

	//TEST
	System.out.println ("Query string:  [" + queryStr.toString() + "]");

        return  (queryStr.toString());
    }

    public boolean action (Event evt, Object arg)
    {
        if (evt.target instanceof Button)
        {
            String  label = (String)arg;
            if (label.equals (QUIT_BUTTON_LABEL))
            {
                hide();
                parent.closeWebSearchWin();
                return  true;
            }
            else  if (label.equals (EXEC_BUTTON_LABEL))
            {

	      if (context == null) {
		String msg = "A web query can only be issued when run " +
		/* Need a frame object if we wish to pop-up a notification */
		// Notification note;
		  "as an applet in a browser.";
		// note = new Notification(parent, "Error", msg);
		// note.showWorkaround();
		System.err.println(msg);
		return true;
	      }

                String  urlAndQStr = getWebQuery();
                try    {
                    URL     webPg = new URL (urlAndQStr);
                    //System.out.println ("Web Query URL:  "
                    //      + webPg.toString());

                    context.showDocument (webPg, WEB_TARGET);
                }
                catch  (MalformedURLException  mue)  {
                    System.out.println ("Problem with query: Cannot retreive");
                }

                return  true;
            }
            else  if (label.equals (THES_BUTTON_LABEL))
            {
                //create new one only if one does not already exist
                if (doWebHelp == null)
                {
                    String  myTitle = getTitle();
                    doWebHelp = new WebQHelp ((Frame)parent,
                             (WebSearchHelpContainer)this, myTitle + " Help");

                    //is 'show()' in Window class or Dialog class?
                    if (ApplType.getIsVersion1_0())
                    {
                        // System.out.println ("Version 1.0");
                        ((Window)doWebHelp).show();
                    }
                    else
                    {
                        // System.out.println ("Version 1.1 or higher");
                        doWebHelp.show();
                    }
                }
                else  //bring existing one to top of stack
                    doWebHelp.toFront();

                return  true;
            }
            else  if (label.equals (COPY_BUTTON_LABEL))
            {
                String  webQStr = qStrGenerator.makeWebQuery();
                webQuery.setText (webQStr);

                return  true;
            }
        }
        return  false;
    }

    /* implements WebSearchHelpContainer interface */
    public void closeWebSearchQHelpWin()
    {
        if (doWebHelp != null)
        {
            doWebHelp.dispose();
            doWebHelp = null;
        }
    }

    /* brings this window's dependent window(s) to front in display */
    public void assocWinToFront()
    {
        if (doWebHelp != null)
            doWebHelp.toFront();
    }

    /* implements WebSearchHelpContainer interface */
    public void writeToSearchArea (String[]  thesaurusWords)
    {
        if (thesaurusWords == null)
            return;

        for (int i = 0; i < thesaurusWords.length; i++)
        {
            boolean  multWords = false;

            webQuery.appendText (" ");

            String  phrase = (thesaurusWords[i]).trim();
            if (phrase.indexOf(' ') >= 0)
            {
                multWords = true;
                webQuery.appendText (String.valueOf('"'));
            }
            webQuery.appendText (thesaurusWords[i]);

            if (multWords)
                webQuery.appendText (String.valueOf('"'));
        }
    }

  /** Allow other objects to set the query string */
  public void setQueryString(String webQStr) { 

    webQuery.setText (webQStr);
  }

}
