/**                   GridBagCreator                       */

package UserInt;

import    java.awt.*;

/**
    The purpose of this class is to make using the GridBag layout and the
    associated GridBagConstraints class easy to use.

    GridBag's cells are not necessarily equal size as they are for the
    GridLayout.  This can lead to confusing results for the user when
    GridBags are nested.  The cell size is related to the weight for each
    dimension.
*/
public class  GridBagCreator  extends Object
{
    GridBagLayout          gridBag;
    GridBagConstraints     gbc;
    Container              container;  // object using grid bag layout

    public  GridBagCreator (Container con)
    {
        gridBag = new GridBagLayout();
        gbc = new GridBagConstraints();
        container = con;
        container.setLayout (gridBag);
    }

    private void setGridBagConstraints (
              int gx, int gy, int gwid, int ght, double wtx, double wty,
              int fill, int xinset, int yinset)
    {
        gbc.gridx = gx;          // which cell, horizontally (0 is first one)
        gbc.gridy = gy;          // which cell, vertically (0 is first one)
        gbc.gridwidth = gwid;    // number of cells spanned, horizontally
        gbc.gridheight = ght;    // number of cells spanned, vertically
        gbc.weightx = wtx;       // how to distribute extra horizontal space
        gbc.weighty = wty;       // how to distribute extra vertical space
        gbc.fill = fill;         // how to fill the cell(s)
        gbc.insets = new Insets (yinset, xinset, yinset, xinset);
		         // external padding for component getting constraints
    }

    /* Using the values given for 'gbc', component 'c' is added to 'container'.
    */
    public void addComponent (Component c,
              int gx, int gy, int gwid, int ght, double wtx, double wty,
              int fill, int xinset, int yinset)
    {
        setGridBagConstraints (gx, gy, gwid, ght, wtx, wty, fill,
                                            xinset, yinset);
        gridBag.setConstraints (c, gbc);
        container.add (c);
    }

    /* default is CENTER
       Possible other values:  NORTH, NORTHEAST, EAST, SOUTHEAST, SOUTH,
            SOUTHWEST, WEST, NORTHWEST.
       Since this is not as commonly used, it is a separate routine.
    */
    public void setAnchor (int newSetting)
    {
        gbc.anchor = newSetting;
    }
}
